/***************************************************************************
 *   Copyright (C) 2009 - 2010 by Dr. Oliver Heidbüchel                    *
 *                         (post@oliver-heidbuechel.de)                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 ***************************************************************************/

#include "statementwidget.h"

#include <Soprano/Soprano>
#include <Soprano/Error/Error>

#include <KLineEdit>
#include <KLocale>
#include <KMessageBox>

#include <QBoxLayout>
#include <QPushButton>
#include <QLabel>
#include <QString>
#include <QStringList>
#include <QUrl>
#include <QWidget>

 
StatementWidget::StatementWidget(Soprano::Model *m, QWidget *parent)    
    : QWidget(parent)
{
  // Model
  model = m;
  
  // Layout
  QVBoxLayout * layout = new QVBoxLayout(this);
  QLabel * subject = new QLabel(i18n("Subject:"), this);
  layout->addWidget(subject);
  subjectField = new KLineEdit(this);
  layout->addWidget(subjectField);
  QLabel * predicate = new QLabel(i18n("Predicate:"), this);
  layout->addWidget(predicate);
  predicateField = new KLineEdit(this);
  layout->addWidget(predicateField);
  QLabel * object = new QLabel(i18n("Object:"), this);
  layout->addWidget(object);
  objectField = new KLineEdit(this);
  layout->addWidget(objectField);
  QLabel * context = new QLabel(i18n("Context:"), this);
  layout->addWidget(context);
  contextField = new KLineEdit(this);
  layout->addWidget(contextField);
  QPushButton * add = new QPushButton(i18n("Add Statement"), this);
  layout->addWidget(add);
  QPushButton * remove = new QPushButton(i18n("Remove Statements"), this);
  layout->addWidget(remove);
  layout->addStretch();
  
  // Connections
  connect(add, SIGNAL(clicked()), this, SLOT(addStatement()));
  connect(remove, SIGNAL(clicked()), this, SLOT(removeStatements()));
}

void StatementWidget::setSubject(const QString &text){
  subjectField->setText(text);
}

void StatementWidget::setPredicate(const QString &text){
  predicateField->setText(text);
}

void StatementWidget::setObject(const QString &text){
  objectField->setText(text);
}

void StatementWidget::setContext(const QString &text){
  contextField->setText(text);
}

void StatementWidget::addStatement(){
  Soprano::Node subject = createNodeForStatement(subjectField->text());
  Soprano::Node predicate = createNodeForStatement(predicateField->text());
  Soprano::Node object = createNodeForStatement(objectField->text());
  Soprano::Node context = createNodeForStatement(contextField->text());
  Soprano::Error::ErrorCode error = model->addStatement(Soprano::Statement(subject, predicate, object, context));

  if (error != Soprano::Error::ErrorNone){
    KMessageBox::error(this, i18n("Adding statement failed:<br>") + model->lastError().message(), i18n("Adding statement failed!"));
  }
  else {
    KMessageBox::error(this, i18n("Added statement \"%1 %2 %3\" with context %4.", subject.toN3(), predicate.toN3(), object.toN3(), context.toN3()),
                       i18n("Added Statement successfully!"));    
  }
}

void StatementWidget::removeStatements(){
  Soprano::Node subject = createNodeForStatement(subjectField->text());
  Soprano::Node predicate = createNodeForStatement(predicateField->text());
  Soprano::Node object = createNodeForStatement(objectField->text());
  Soprano::Node context = createNodeForStatement(contextField->text());
  QString s = subject.toN3();
  QString p = predicate.toN3();
  QString o = object.toN3();
  QString c = context.toN3();
  if (s == "") s = "*";
  if (p == "") p = "*";
  if (o == "") o = "*";
  if (c == "") c = "*";

  QString table = "<table><tr><td>" +
                  i18n("subject: ") + "</td><td>" + bracketsToHTML(s) + "</td></tr><tr><td>" +
                  i18n("predicate: ") + "</td><td>" + bracketsToHTML(p) + "</td></tr><tr><td>" + 
                  i18n("object: ") + "</td><td>" + bracketsToHTML(o) + "</td></tr><tr><td>" + 
                  i18n("context: ") + "</td><td>" + bracketsToHTML(c) + "</td></tr></table>";   
    
  if (KMessageBox::warningContinueCancel(this, i18n("Do you really want to remove all statements with<br>") + table, 
                                          i18n("Really remove?")) == KMessageBox::Cancel){
      return;
  }
  Soprano::Error::ErrorCode error = model->removeAllStatements(Soprano::Statement(subject, predicate, object, context));

  if (error != Soprano::Error::ErrorNone){
    KMessageBox::error(this, i18n("Removing statement failed: ") + model->lastError().message(), i18n("Removing statement failed!"));
  }
  else {
    KMessageBox::error(this, i18n("Removed all statements with<br>") + table, i18n("Removed Statements successfully!"));    
  }
}

QString StatementWidget::removeBrackets(const QString &text){
  QString shorter = text.trimmed();
  if (shorter.startsWith("<") && shorter.endsWith(">")){
    return (shorter.mid(1, shorter.count() - 2));
  }
  else {
    return(shorter);
  }
}

QString StatementWidget::bracketsToHTML(const QString &text){
  QString s(text);
  s.replace(QString("<"), QString("&lt;"));
  s.replace(QString(">"), QString("&gt;"));
  return(s);
}

QString StatementWidget::removeQuotationMarks(const QString &text){
  QString shorter = text.trimmed();
  if (shorter.startsWith("\"") && shorter.endsWith("\"")){
    return (shorter.mid(1, shorter.count() - 2));
  }
  else {
    return(shorter);
  }
}

Soprano::Node StatementWidget::createNodeForStatement(const QString &text){
  QString wb = removeBrackets(text);
  if (wb == ""){
    return(Soprano::Node::createEmptyNode());
  }
  else if(wb.startsWith("\"")){
    QStringList list = wb.split("^^");
    Soprano::LiteralValue lv = Soprano::LiteralValue::fromString(removeQuotationMarks(list.at(0)), QUrl(removeBrackets(list.at(1))));
    return(Soprano::Node::createLiteralNode(lv));
  }
  else{
    return(Soprano::Node::createResourceNode(wb));
  }
}








